# Copyright 2019 (c) Anna Schumaker.
from . import playlist
from .. import notify
from .. import threadqueue
from .. import trak
from .. import tree
import os

LIBRARY_ICON   =   "folder-music"
LIBRARY_SORT   = [ "artist", "date", "album", "discnumber", "tracknumber" ]
library_thread = threadqueue.ThreadQueue()

class LibraryPlaylist(playlist.Playlist):
    def scan(self):
        library_thread.push(self.thread_scan)

    def thread_add(self, path):
        self.add(trak.lookup(path))

    def thread_scan(self):
        for dirname, subdirs, files in os.walk(self.name):
            for f in files:
                library_thread.push(self.thread_add, os.path.join(dirname, f))
            library_thread.push(notify.notify, "save-data")


class LibraryNode(tree.ETree):
    def __init__(self):
        tree.ETree.__init__(self, "Libraries", LIBRARY_ICON)

    def alloc_child(self, path):
        return LibraryPlaylist(path, LIBRARY_ICON, LIBRARY_SORT)

    def lookup(self, path):
        library = tree.ETree.lookup(self, os.path.abspath(path))
        library.scan()
        return library


def join():
    library_thread.join()

def reset():
    global library_thread
    if not library_thread.is_alive():
        library_thread = threadqueue.ThreadQueue()

def stop():
    library_thread.stop()
