# Copyright 2019 (c) Anna Schumaker.
from . import playlist
from .. import notify

COLL_ICON =   "media-playback-start"
COLL_SORT = [ "artist", "date", "album", "discnumber", "tracknumber" ]

NEXT_ICON =   "edit-redo"
NEXT_SORT = [ ]

PREV_ICON =   "edit-undo"
PREV_SORT = [ ]


class CollectionPlaylist(playlist.Playlist):
    def __init__(self):
        playlist.Playlist.__init__(self, "Collection", COLL_ICON, COLL_SORT, can_loop=False)
        self.loop = True

    def new_track(self, track):
        self.add(track)

    def reset(self):
        playlist.Playlist.reset(self)
        self.loop = True


class PreviousPlaylist(playlist.Playlist):
    def __init__(self):
        playlist.Playlist.__init__(self, "Previous", PREV_ICON, PREV_SORT,
                                          can_loop=False, can_random=False)

    def __getstate__(self):
        state = playlist.Playlist.__getstate__(self)
        state["list"] = [ ]
        state["current"] = -1
        return state

    def add(self, track):
        if track is not None:
            self.remove(track)
            with self.plist_lock:
                self.list.insert(0, track)
                self.current = 0
            self.track_added(track, 0)

    def sort(self, field):
        return False


class UpNextPlaylist(playlist.Playlist):
    def __init__(self):
        playlist.Playlist.__init__(self, "Up Next", NEXT_ICON, NEXT_SORT, can_loop=False)

    def next(self):
        track = playlist.Playlist.next(self)
        self.remove(track)
        if len(self) == 0:
            self.reset()
        return track

    def track_removed(self, track, index):
        notify.notify("remove-track", self, track, index)
        self.changed()
