# Copyright 2019 (c) Anna Schumaker.
from . import playlist
from . import special
from .. import notify
from .. import trak
import os
import unittest

test_album = os.path.abspath("./trier/Test Album")

class TestCollectionPlaylist(unittest.TestCase):
    def test_collection_plist(self):
        plist = special.CollectionPlaylist()

        self.assertIsInstance(plist,  special.CollectionPlaylist)
        self.assertIsInstance(plist,  playlist.Playlist)
        self.assertEqual(plist.name, "Collection")
        self.assertEqual(plist.icon, "media-playback-start")
        self.assertEqual(plist.sort_order,
                    [ "artist", "date", "album", "discnumber", "tracknumber" ])

    def test_collection_new_track(self):
        plist = special.CollectionPlaylist()
        self.assertEqual(len(plist),  0)

        track1 = trak.lookup(os.path.join(test_album, "01 - Test Track.ogg"))
        self.assertEqual(len(plist),  1)
        self.assertEqual(plist[0],    track1)

    def test_collection_reset(self):
        plist = special.CollectionPlaylist()
        plist.list.append(42)
        plist.random = True
        plist.loop   = False
        notify.cancel("new-track", plist.new_track)

        plist.reset()
        self.assertEqual(len(plist), 0)
        self.assertFalse(plist.random)
        self.assertTrue(plist.loop)
        self.assertIn((plist.new_track, False), notify.events["new-track"])

    def test_collection_loop(self):
        plist = special.CollectionPlaylist()
        self.assertFalse(plist.can_loop)
        self.assertTrue( plist.loop)
        self.assertTrue( plist.set_loop(False))
        self.assertTrue( plist.loop)
