# Copyright 2019 (c) Anna Schumaker.
from . import decade
from . import playlist
from .. import notify
from .. import trak
from .. import tree
import os
import unittest

test_library = os.path.abspath("./trier/Test Library/Test Artist 01")
test_album   = os.path.join("Test Album 1", "01 - Test Track 01.ogg")
test_album2  = os.path.join("Test Album 4", "01 - Test Track 01.ogg")
test_album3  = os.path.join("Test Album 4", "02 - Test Track 02.ogg")

class TestDecadePlaylist(unittest.TestCase):
    def setUp(self):
        notify.clear()
        trak.reset()

    def test_decade_node(self):
        dnode = decade.DecadeNode()
        self.assertIsInstance(dnode, tree.ETree)
        self.assertEqual(dnode.name, "Decades")
        self.assertEqual(dnode.icon, "x-office-calendar")

    def test_decade_alloc_child(self):
        dnode = decade.DecadeNode()
        plist = dnode.alloc_child("1980s")
        self.assertIsInstance(plist, decade.DecadePlaylist)
        self.assertEqual(plist.name, "1980s")
        self.assertEqual(plist.icon, "x-office-calendar")
        self.assertEqual(plist.sort_order,
                    [ "date", "artist", "album", "discnumber", "tracknumber"])

    def test_decade_node_new_track(self):
        dnode = decade.DecadeNode()
        self.assertEqual(dnode.n_children(), 0)

        track = trak.lookup(os.path.join(test_library, test_album))
        self.assertEqual(dnode.n_children(), 1)
        plist = dnode.nth_child(0)
        self.assertEqual(plist.name, "1970s")
        self.assertEqual(plist.icon, "x-office-calendar")
        self.assertEqual(plist[0], track)

    def test_decade_node_reset(self):
        dnode = decade.DecadeNode()
        dnode.children.append(42)
        notify.cancel("new-track", dnode.new_track)

        dnode.reset()
        self.assertEqual(len(dnode.children), 0)
        self.assertIn((dnode.new_track, False), notify.events["new-track"])

    def test_decade_playlist_alloc(self):
        dnode = decade.DecadePlaylist("Test Decade", decade.DECADE_ICON)
        year  = dnode.alloc_child("1973")

        self.assertIsInstance(year, playlist.Playlist)
        self.assertEqual(year.name, "1973")
        self.assertEqual(year.icon, "x-office-calendar")
        self.assertEqual(year.sort_order,
                        [ "artist", "album", "discnumber", "tracknumber" ])

    def test_decade_playlist_add(self):
        dnode = decade.DecadeNode()
        track = trak.lookup(os.path.join(test_library, test_album))
        plist = dnode.nth_child(0)

        self.assertEqual(plist.n_children(), 1)
        year = plist.nth_child(0)
        self.assertIsInstance(year, playlist.Playlist)
        self.assertEqual(year.name, "1973")
