# Copyright 2019 (c) Anna Schumaker.
from . import playlist
from . import special
from .. import notify
from .. import trak
import os
import unittest

test_album = os.path.abspath("./trier/Test Library/Test Artist 01/Test Album 1")

class TestPreviousPlaylist(unittest.TestCase):
    def setUp(self):
        notify.register("add-track", self.on_add_track)
        self.cb_plist = None
        self.cb_track = None
        self.cb_index = 0

    def tearDown(self):
        notify.cancel("add-track", self.on_add_track)

    def on_add_track(self, plist, track, index):
        if isinstance(plist, special.PreviousPlaylist):
            self.cb_plist = plist
            self.cb_track = track
            self.cb_index = index

    def test_previous_playlist(self):
        plist = special.PreviousPlaylist()
        self.assertIsInstance(plist, special.PreviousPlaylist)
        self.assertIsInstance(plist, playlist.Playlist)
        self.assertEqual(plist.name, "Previous")
        self.assertEqual(plist.icon, "edit-undo")

    def test_previous_getstate(self):
        plist = special.PreviousPlaylist()
        track = trak.lookup(os.path.join(test_album, "01 - Test Track 01.ogg"))

        plist.add(track)
        plist.current = 3

        state = plist.__getstate__()
        self.assertEqual(state["name"], "Previous")
        self.assertEqual(state["icon"], "edit-undo")
        self.assertEqual(state["list"], [ ])
        self.assertEqual(state["current"], -1)

    def test_previous_add(self):
        plist = special.PreviousPlaylist()
        plist.visible = True

        track1 = trak.lookup(os.path.join(test_album, "01 - Test Track 01.ogg"))
        track2 = trak.lookup(os.path.join(test_album, "02 - Test Track 02.ogg"))
        track3 = trak.lookup(os.path.join(test_album, "03 - Test Track 03.ogg"))

        self.assertEqual(plist.current, -1)
        self.assertEqual(plist.next(),  None)

        plist.add(track1)
        self.assertEqual(self.cb_plist, plist)
        self.assertEqual(self.cb_track, track1)
        self.assertEqual(self.cb_index, 0)
        self.assertEqual(plist.current, 0)
        self.assertEqual(plist.next(),  None)

        plist.add(track2)
        self.assertEqual(self.cb_plist, plist)
        self.assertEqual(self.cb_track, track2)
        self.assertEqual(self.cb_index, 0)
        self.assertEqual(plist.current, 0)

        self.assertEqual(len(plist), 2)
        self.assertEqual(plist[0], track2)
        self.assertEqual(plist[1], track1)

        self.assertEqual(plist.next(),  track1)
        self.assertEqual(plist.current, 1)

        plist.add(track3)
        self.assertEqual(plist.current, 0)

        plist.add(track1)
        self.assertEqual(len(plist), 3)
        self.assertEqual(plist[0], track1)
        self.assertEqual(plist[1], track3)
        self.assertEqual(plist[2], track2)

    def test_previous_loop(self):
        plist = special.PreviousPlaylist()
        self.assertFalse(plist.can_loop)
        self.assertFalse(plist.loop)
        self.assertFalse(plist.set_loop(True))
        self.assertFalse(plist.loop)

    def test_previous_random(self):
        plist = special.PreviousPlaylist()
        self.assertFalse(plist.can_random)
        self.assertFalse(plist.random)
        self.assertFalse(plist.set_random(True))
        self.assertFalse(plist.random)

    def test_previous_sort(self):
        plist = special.PreviousPlaylist()
        self.assertEqual(plist.sort_order, [ ])
        self.assertFalse(plist.sort("date"))
        self.assertEqual(plist.sort_order, [ ])
