# Copyright 2019 (c) Anna Schumaker.
from . import artist
from . import decade
from . import genre
from . import library
from . import playlist
from . import root
from . import special
from . import user
from .. import notify
from .. import threadqueue
from .. import trak
from .. import tree
import os
import random
import unittest

test_album   = os.path.abspath("./trier/Test Album")
test_library = os.path.abspath("./trier/Test Library")

class TestPlaylistRoot(unittest.TestCase):
    def setUp(self):
        trak.reset()
        library.reset()
        notify.register("next-track",       self.on_next_track)
        notify.register("playlist-changed", self.on_playlist_changed)

        try:
            self.playman.reset()
        except:
            self.playman  = root.PlaylistRoot()
        self.cb_plist  = None
        self.cb_index  = None
        self.cb_first  = None
        self.cb_track  = None
        self.cb_change = [ ]

    def tearDown(self):
        notify.cancel("new-track",  self.playman.lookup("Collection").add)
        notify.cancel("new-track",  self.playman.lookup("Genres").new_track)
        notify.cancel("next-track", self.on_next_track)
        notify.register("playlist-changed", self.on_playlist_changed)
        library.stop()

    def on_playlist_changed(self, plist):
        self.cb_change.append(plist)

    def on_next_track(self, track):
        self.cb_track = track

    def test_root_init(self):
        self.assertIsInstance(self.playman, root.PlaylistRoot)
        self.assertIsInstance(self.playman, tree.ETree)

        self.assertIsInstance(self.playman.lookup("Collection"), special.CollectionPlaylist)
        self.assertIsInstance(self.playman.lookup("Up Next"),    special.UpNextPlaylist)
        self.assertIsInstance(self.playman.lookup("Previous"),   special.PreviousPlaylist)
        self.assertIsInstance(self.playman.lookup("Playlists"),  user.UserNode)
        self.assertIsInstance(self.playman.lookup("Artists"),    artist.ArtistNode)
        self.assertIsInstance(self.playman.lookup("Genres"),     genre.GenreNode)
        self.assertIsInstance(self.playman.lookup("Decades"),    decade.DecadeNode)
        self.assertIsInstance(self.playman.lookup("Libraries"),  library.LibraryNode)

        self.assertEqual(self.playman.nth_child(0), self.playman.lookup("Collection"))
        self.assertEqual(self.playman.nth_child(1), self.playman.lookup("Up Next"))
        self.assertEqual(self.playman.nth_child(2), self.playman.lookup("Previous"))
        self.assertEqual(self.playman.nth_child(3), self.playman.lookup("Playlists"))
        self.assertEqual(self.playman.nth_child(4), self.playman.lookup("Artists"))
        self.assertEqual(self.playman.nth_child(5), self.playman.lookup("Genres"))
        self.assertEqual(self.playman.nth_child(6), self.playman.lookup("Decades"))
        self.assertEqual(self.playman.nth_child(7), self.playman.lookup("Libraries"))

        for i, plist in enumerate(self.playman.children):
            self.assertEqual(plist.sibling, self.playman.nth_child(i + 1))

        self.assertEqual(self.playman.name,   "root")

    def test_root_on_scan(self):
        plist = self.playman.lookup("Libraries").lookup(test_library)
        alist = self.playman.lookup("Libraries").lookup(test_album)
        library.join()

        self.assertEqual(len(plist), 1250)
        self.assertEqual(len(alist), 12)
        self.assertEqual(len(self.playman.lookup("Collection")), len(alist) + len(plist))

        self.playman.reset()
        self.assertEqual(len(self.playman.lookup("Collection")), 0)
