# Copyright 2019 (c) Anna Schumaker.
from . import user
from .. import trak
from .. import tree
import unittest

class TestUserPlaylists(unittest.TestCase):
    def test_user_node(self):
        unode = user.UserNode()

        self.assertIsInstance(unode, tree.ETree)
        self.assertEqual(unode.name, "Playlists")
        self.assertEqual(unode.icon, "audio-x-generic")

        self.assertEqual(unode.n_children(), 2)
        self.assertEqual(unode.nth_child(0).name, "New Tracks")
        self.assertEqual(unode.nth_child(0).icon, user.NEW_ICON)
        self.assertEqual(unode.nth_child(1).name, "Starred")
        self.assertEqual(unode.nth_child(1).icon, user.STAR_ICON)

        self.assertIsInstance(unode.lookup("New Tracks"), user.NewPlaylist)

    def test_user_reset(self):
        unode = user.UserNode()
        new   = unode.lookup("New Tracks")
        star  = unode.lookup("Starred")

        unode.reset()
        self.assertEqual(unode.n_children(), 2)

        self.assertNotEqual(unode.nth_child(0), new)
        self.assertEqual(unode.nth_child(0).name, "New Tracks")
        self.assertEqual(unode.nth_child(0).icon, user.NEW_ICON)

        self.assertNotEqual(unode.nth_child(1), star)
        self.assertEqual(unode.nth_child(1).name, "Starred")
        self.assertEqual(unode.nth_child(1).icon, user.STAR_ICON)

    def test_user_starred(self):
        unode = user.UserNode()
        star  = unode.lookup("Starred")

        self.assertEqual(star.name, "Starred")
        self.assertEqual(star.icon, "starred")
        self.assertEqual(star.sort_order,
                    [ "artist", "date", "album", "discnumber", "tracknumber" ])
