# Copyright 2019 (c) Anna Schumaker.
from . import threadqueue
import queue
import unittest
import threading

test_a = 0
test_b = 0
test_c = 0
test_l = threading.Lock()

def set_abc(a, b, c):
    global test_a, test_b, test_c
    with test_l:
        test_a = a
        test_b = b
        test_c = c

class TestThreadQueue(unittest.TestCase):
    def setUp(self):
        set_abc(0, 0, 0)

    def test_threadqueue_init(self):
        tq = threadqueue.ThreadQueue()
        self.assertIsInstance(tq, queue.Queue)
        self.assertIsInstance(tq, threading.Thread)
        self.assertIsInstance(tq.stop_event, threading.Event)
        self.assertTrue(tq.is_alive())
        tq.stop()
        self.assertFalse(tq.is_alive())

    def test_threadqueue_push(self):
        tq = threadqueue.ThreadQueue()
        with test_l:
            tq.push(set_abc, 1, 2, 3)

        tq.join()
        self.assertEqual(test_a, 1)
        self.assertEqual(test_b, 2)
        self.assertEqual(test_c, 3)

        for i in range(1000):
            tq.push(set_abc, i, i+1, i+2)

        tq.join()
        self.assertEqual(test_a, 999)
        self.assertEqual(test_b, 1000)
        self.assertEqual(test_c, 1001)
        tq.stop()
