# Copyright 2019 (c) Anna Schumaker.
from .. import data
from .. import notify
from . import track
import threading

trak_file = "tracks.pickle"
trak_lock = threading.Lock()
trak_db   = dict()

def clear():
    with trak_lock:
        trak_db.clear()

def exists():
    with trak_lock:
        return data.DataFile(trak_file, data.READ).exists()

def load():
    with trak_lock:
        with data.DataFile(trak_file, data.READ) as f:
            trak_db.update(f.unpickle())

def lookup(path):
    with trak_lock:
        ret = trak_db.get(path)
        if ret == None:
            try:
                ret = track.Track(path)
                trak_db[path] = ret
                notify.notify("new-track", ret)
            except:
                return None
    return ret

def remove():
    with trak_lock:
        return data.DataFile(trak_file, data.WRITE).remove()

def reset():
    with trak_lock:
        trak_db.clear()
        data.DataFile(trak_file, data.WRITE).remove()
        notify.register("save-data", save)

def save():
    with trak_lock:
        with data.DataFile(trak_file, data.WRITE) as f:
            f.pickle(trak_db)

notify.register("save-data", save)
