# Copyright 2019 (c) Anna Schumaker.
import mutagen
import re

discno_map = { "one" : "1", "two" : "2", "three" : "3", "four" : "4", "five" : "5" }


class TagFile:
    def __init__(self, f):
        self.tag = mutagen.File(f)

    def album(self):
        a = self.get("album", "Unknown Album")
        m = re.search("(cd|dis[c|k])(\s)*(([0-9]+)|one|two|three|four|five)", a.lower())
        if m and m.start() > 0:
            return a[:m.start()].strip(" ;({[-")
        return a

    def album_artist(self):
        for tag in [ "albumartist", "album artist", "artist" ]:
            if tag in self.tag:
                return self.tag[tag][0]
        return "Unknown Artist"

    def discno(self):
        d = self.get("album", "Unknown Album")[len(self.album()):]
        m = re.search("([1-9][0-9]*)|one|two|three|four|five", d.lower())
        if m:
            return int(discno_map.get(m.group(), m.group()))
        return self.get_int("discnumber", 1)

    def get(self, tag, fallback):
        return self.tag.get(tag, [ fallback ])[0]

    def get_int(self, tag, fallback):
        return int(re.match("\d+", self.get(tag, str(fallback))).group(0))

    def length(self):
        return self.tag.info.length


class FakeFile(TagFile):
    def __init__(self, tags={}):
        self.tag = tags
