# Copyright 2019 (c) Anna Schumaker.
from . import tagfile
import mutagen
import os
import unittest

test_album = os.path.abspath("./trier/Test Album")

class TestMutagen(unittest.TestCase):
    def test_tagfile(self):
        f = tagfile.TagFile(os.path.join(test_album, "01 - Test Track.ogg"))
        self.assertIsInstance(f.tag, mutagen.oggvorbis.OggVorbis)

        self.assertEqual(f.get("title", None),       "Test Track")
        self.assertEqual(f.get("notag", "Fallback"), "Fallback")

        self.assertEqual(f.get_int("date",     None),        2019)
        self.assertEqual(f.get_int("someint", "2019-11-22"), 2019)

        self.assertEqual(f.length(), f.tag.info.length)

    def test_fakefile(self):
        f = tagfile.FakeFile({ "title" : [ "Test Title" ]})
        self.assertIsInstance(f, tagfile.TagFile)
        self.assertEqual(f.get("title", "Fallback"), "Test Title")

    def test_album_artist(self):
        f = tagfile.FakeFile({ "albumartist" : [ "1" ], "album artist" : [ "2" ], "artist" : [ "3" ]})
        self.assertEqual(f.album_artist(), "1")
        f.tag.pop("albumartist")
        self.assertEqual(f.album_artist(), "2")
        f.tag.pop("album artist")
        self.assertEqual(f.album_artist(), "3")
        f.tag.pop("artist")
        self.assertEqual(f.album_artist(), "Unknown Artist")

    def test_discno_detect(self):
        albums = [ "Test Album {Disc 1}",   "Test Album [Disk One]",
                   "Test Album (Disk Two)", "Test Album - Disc Three",
                   "Test Album;CD Four",    "Test Album;CdFive",
                   "Test Album CD 9/10" ]
        discs  = [ 1, 1, 2, 3, 4, 5, 9 ]

        self.assertEqual(tagfile.FakeFile({ }).album(), "Unknown Album")

        for i, a in enumerate(albums):
            f = tagfile.FakeFile({ "album" : [ a ]})
            self.assertEqual(f.album(), "Test Album")
            self.assertEqual(f.discno(), discs[i])

        f = tagfile.FakeFile({ "album" : [ "Disc One: Test Album" ]})
        self.assertEqual(f.album(), "Disc One: Test Album")
        self.assertEqual(f.discno(), 1)
