# Copyright 2019 (c) Anna Schumaker.
from . import track
from .. import notify
import os
import unittest

test_tracks = os.path.abspath("./trier/Test Album")

class TestTrackTag(unittest.TestCase):
    def setUp(self):
        notify.register("new-track", self.on_new_track)
        self.cb_track = None

    def tearDown(self):
        notify.cancel("new-track", self.on_new_track)

    def on_new_track(self, track):
        self.cb_track = track

    def test_track_init_basic(self):
        p = [ test_tracks, "01 - Test Track.ogg" ]
        t = track.Track(os.path.join(*p))
        self.assertEqual(t.path,        os.path.join(*p))
        self.assertEqual(t.album,       "Test Album")
        self.assertEqual(t.albumartist, "Test Artist")
        self.assertEqual(t.artist,      "Test Artist")
        self.assertEqual(t.genre,       "Test")
        self.assertEqual(t.title,       "Test Track")

        self.assertEqual(t.date,         2019)
        self.assertEqual(t.discnumber,   1)
        self.assertEqual(t.length,       10)
        self.assertEqual(t.tracknumber,  1)
        self.assertEqual(t.tracktotal,   12)

    def test_track_init_empty(self):
        p = [ test_tracks, "00 - Empty Track.ogg" ]
        t = track.Track(os.path.join(*p))
        self.assertEqual(t.path,        os.path.join(*p))
        self.assertEqual(t.album,       "Unknown Album")
        self.assertEqual(t.albumartist, "Unknown Artist")
        self.assertEqual(t.artist,      "Unknown Artist")
        self.assertEqual(t.genre,       "Unknown")
        self.assertEqual(t.title,       "00 - Empty Track.ogg")

        self.assertEqual(t.date,        0)
        self.assertEqual(t.discnumber,  1)
        self.assertEqual(t.length,      0)
        self.assertEqual(t.tracknumber, 0)
        self.assertEqual(t.tracktotal,  0)

    def test_track_decade(self):
        t = track.Track(os.path.join(test_tracks, "01 - Test Track.ogg"))
        self.assertEqual(t.decade(), "2010s")
        t.date = 1975
        self.assertEqual(t.decade(), "1970s")

    def test_track_getitem(self):
        p = [ test_tracks, "01 - Test Track.ogg" ]
        t = track.Track(os.path.join(*p))
        self.assertEqual(t["path"],         os.path.join(*p))
        self.assertEqual(t["title"],       "Test Track")
        self.assertEqual(t["artist"],      "Test Artist")
        self.assertEqual(t["tracknumber"], "01")
        self.assertEqual(t["discnumber"],  "Disc 1")
        self.assertEqual(t["length"],      "0:10")
        self.assertEqual(t["album"],       "Test Album")
        self.assertEqual(t["date"],        "2019")
        self.assertEqual(t["genre"],       "Test")
        self.assertEqual(t["albumartist"], "Test Artist")
        self.assertIsNone(t["nokey"])

    def test_track_sort_key(self):
        t = track.Track(os.path.join(test_tracks, "01 - Test Track.ogg"))
        self.assertEqual(t.sort_key(["title"]), [[ "test", "track" ]])
        self.assertEqual(t.sort_key(["title", "tracknumber"]), [ ["test", "track"], 1 ])

        t.title = "The Test - Track!"
        self.assertEqual(t.sort_key(["title"]), [[ "test", "track" ]])
        t.title = "A \"Test\" Track"
        self.assertEqual(t.sort_key(["title"]), [[ "test", "track" ]])
        t.title = "Test Träck 01"
        self.assertEqual(t.sort_key(["title"]), [[ "test", "track", "01" ]])
