# Copyright 2019 (c) Anna Schumaker.
from . import tagfile
from .. import notify
from .. import sort
import os
import sys

class Track:
    def __init__(self, path):
        tag = tagfile.TagFile(path)

        self.path        = path
        self.album       = tag.album()
        self.albumartist = tag.album_artist()
        self.artist      = tag.get("artist", "Unknown Artist")
        self.genre       = tag.get("genre",  "Unknown")
        self.title       = tag.get("title",   os.path.basename(path))
        self.date        = tag.get_int("date",        0)
        self.tracknumber = tag.get_int("tracknumber", 0)
        self.tracktotal  = tag.get_int("tracktotal",  0)
        self.discnumber  = tag.discno()
        self.length      = tag.length()

    def __getitem__(self, key):
        if key == "tracknumber":
            return f"{self.tracknumber:02}"
        elif key == "discnumber":
            return f"Disc {self.discnumber}"
        elif key == "length":
            (m, s) = divmod(int(self.length), 60)
            return f"{m}:{s:02}"
        item = vars(self).get(key)
        return str(item) if item != None else None

    def decade(self):
        return f"{self.date // 10 * 10}s"

    def sort_key(self, fields):
        ret = [ ]
        for field in fields:
            val = vars(self).get(field)
            if isinstance(val, str):
                val = sort.key(val)
            ret.append(val)
        return ret
