# Copyright 2019 (c) Anna Schumaker.
from . import playbin
from .. import gtk
import curds
import re
from gi.repository import GLib, Gtk, Gst

MINIMUM = -1
MAXIMUM = 99

Box     = gtk.Builder.get_object("pause_box")
Count   = MINIMUM
Down    = gtk.Builder.get_object("pause_down")
Entry   = gtk.Builder.get_object("pause_entry")
Icon    = Gtk.EntryIconPosition.PRIMARY
Label   = gtk.Builder.get_object("pause_estimate")
Popover = gtk.Builder.get_object("pause_popover")
Up      = gtk.Builder.get_object("pause_up")

Playing = False
PopCB   = None


def __changed__():
    Entry.set_text(__text__())
    Entry.set_icon_from_icon_name(Icon, __icon__())
    Up.set_sensitive(Count < MAXIMUM)
    Down.set_sensitive(Count > MINIMUM)

def __estimate__():
    tracks  = curds.playlist.peek(Count)
    length  = sum([ t.length for t in tracks if t is not None ])
    length += playbin.remaining() / Gst.SECOND
    (m, s)  = divmod(round(length / 5.0) * 5, 60)
    res     = [ ]
    if m > 0: res.append(f"{m} minute{'s' if m != 1 else ''}")
    if s > 0: res.append(f"{s} seconds")
    return f"About {' '.join(res)}"

def __text__():
    if Count == -1:
        if Playing == True:
            return "Keep Playing"
        return "Paused"
    elif Count == 0:
        return "This Track"
    elif Count == 1:
        return "Next Track"
    return f"{Count} Tracks"

def __icon__():
    if Count == -1 and Playing == True:
        return "media-playback-start"
    return "media-playback-pause"

def __popup__():
    global PopCB
    if PopCB != None:
        GLib.source_remove(PopCB)
    if Count == MINIMUM:
        __popdown__()
        return
    Label.set_text(__estimate__())
    Popover.popup()
    PopCB = GLib.timeout_add_seconds(5, __popdown__)

def __popdown__():
    global PopCB
    PopCB = None
    Popover.popdown()
    return GLib.SOURCE_REMOVE


def on_up_clicked(button):
    global Count
    Count = min(MAXIMUM, Count + 1)
    __changed__()
    __popup__()

Up.connect("clicked", on_up_clicked)


def decrement():
    global Count
    old   = Count
    Count = max(MINIMUM, Count - 1)
    __changed__()
    if Count != old and Count == MINIMUM:
        return Gst.State.PAUSED
    return Gst.State.PLAYING

def on_down_clicked(button):
    decrement()
    __popup__()

Down.connect("clicked", on_down_clicked)


def on_activate_entry(entry):
    global Count
    match = re.search("-?\d+", entry.get_text())
    if match:
        Count = max(-1, min(99, int(match.group(0))))
    __changed__()
    __popup__()

Entry.connect("activate", on_activate_entry)


def on_show_more(visible):
    Box.set_visible(visible)

curds.notify.register("show-more", on_show_more)


def state_changed(state):
    global Playing
    Playing = (state == Gst.State.PLAYING)
    __changed__()

def about_to_pause():
    return Count == (MINIMUM + 1)


def reset():
    global Count
    global Playing
    global PopCB
    Count   = -1
    Playing = False
    __changed__()
    on_show_more(False)
    Popover.hide()
    if PopCB != None:
        GLib.source_remove(PopCB)
        PopCB = None
    curds.notify.register("show-more", on_show_more)
