# Copyright 2019 (c) Anna Schumaker.
from . import artwork
from . import playbin
from . import widgets
from .. import gtk
import os
import unittest
from gi.repository import GdkPixbuf, Gtk, Gst

test_track = os.path.abspath("./trier/Test Album/01 - Test Track.ogg")

class TestAudioArtwork(unittest.TestCase):
    def setUp(self):
        playbin.reset()
        widgets.reset()

    def test_artwork_init(self):
        self.assertIsInstance(artwork.Default,   GdkPixbuf.Pixbuf)
        self.assertIsInstance(artwork.Image,     Gtk.Image)
        self.assertIsInstance(artwork.Paned,     Gtk.Paned)
        self.assertEqual(artwork.IdleCount, 0)
        self.assertEqual(artwork.Pixbuf, artwork.Default)
        self.assertTrue(artwork.Image.is_visible())

    def test_artwork_load(self):
        artwork.Paned.set_position(150)
        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(artwork.IdleCount, 1)
        while (artwork.IdleCount > 0):
            gtk.main_loop()

        self.assertTrue(artwork.Image.is_visible())
        self.assertEqual(artwork.Image.get_storage_type(), Gtk.ImageType.PIXBUF)
        self.assertEqual(artwork.Pixbuf.get_height(), 512)
        self.assertEqual(artwork.Pixbuf.get_width(),  512)

        pbuf = artwork.Image.get_pixbuf()
        self.assertEqual(pbuf.get_height(), 135)
        self.assertEqual(pbuf.get_width(),  135)

        playbin.set_uri(test_track)
        self.assertEqual(artwork.Pixbuf, artwork.Default)

    def test_artwork_resize(self):
        artwork.Paned.set_position(150)
        playbin.set_uri(test_track, Gst.State.PAUSED)
        while (artwork.IdleCount > 0):
            gtk.main_loop()

        pbuf = artwork.Image.get_pixbuf()
        self.assertEqual(pbuf.get_height(), 135)
        self.assertEqual(pbuf.get_width(),  135)

        artwork.Paned.set_position(140)
        pbuf = artwork.Image.get_pixbuf()
        self.assertEqual(pbuf.get_height(), 126)
        self.assertEqual(pbuf.get_width(),  126)
