# Copyright 2019 (c) Anna Schumaker.
from . import playbin
from .. import node
import curds
import os
import unittest
from gi.repository import GLib, Gst

test_album = os.path.abspath("./trier/Test Library/Test Artist 01/Test Album 1")
test_track = os.path.join(test_album, "10 - Test Track 10.ogg")

class TestPlaybin(unittest.TestCase):
    def setUp(self):
        curds.reset()
        playbin.reset()

    def test_playbin_init(self):
        self.assertIsInstance(playbin.Playbin,  Gst.Element)
        self.assertIsInstance(playbin.Fakesink, Gst.Element)
        self.assertIsInstance(playbin.Bus,      Gst.Bus)

        self.assertEqual(playbin.get_property("video-sink"), playbin.Fakesink)

    def test_playbin_set_uri(self):
        playbin.set_uri(test_track)
        uri = GLib.filename_to_uri(test_track)
        self.assertEqual(playbin.get_property("uri"), uri)
        self.assertEqual(playbin.get_state(), Gst.State.NULL)

        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)

    def test_playbin_volume(self):
        self.assertEqual(playbin.get_property("volume"), 1.0)
        playbin.set_volume(50)
        self.assertEqual(playbin.get_property("volume"), 0.5)
        playbin.set_volume(0)
        self.assertEqual(playbin.get_property("volume"), 0.0)

    def test_playbin_play(self):
        playbin.set_uri(test_track)
        self.assertEqual(playbin.play(),      Gst.StateChangeReturn.ASYNC)
        self.assertEqual(playbin.get_state(), Gst.State.PLAYING)

    def test_playbin_pause(self):
        playbin.set_uri(test_track)
        self.assertEqual(playbin.pause(),     Gst.StateChangeReturn.ASYNC)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)

    def test_playbin_duration(self):
        self.assertEqual(playbin.duration(), 0)
        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.duration(), 10 * Gst.SECOND)

    def test_playbin_seek_percent(self):
        self.assertEqual(playbin.position(), 0)
        self.assertEqual(playbin.progress(), 0)

        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(), 0)
        self.assertEqual(playbin.progress(), 0)

        self.assertTrue(playbin.seek_percent(50))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(), 5 * Gst.SECOND)
        self.assertEqual(playbin.progress(), 50)

        self.assertTrue(playbin.seek_percent(75))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(), 7.5 * Gst.SECOND)
        self.assertEqual(playbin.progress(), 75)

        self.assertTrue(playbin.seek_percent(25))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.position(), 2.5 * Gst.SECOND)
        self.assertEqual(playbin.progress(), 25)

    def test_playbin_seek_step(self):
        self.assertEqual(playbin.remaining(), 0)

        playbin.set_uri(test_track, Gst.State.PAUSED)
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.remaining(), 10 * Gst.SECOND)

        self.assertTrue(playbin.seek_step(5))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.remaining(), 5 * Gst.SECOND)

        self.assertTrue(playbin.seek_step(2.5))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.remaining(), 2.5 * Gst.SECOND)

        self.assertTrue(playbin.seek_step(-5))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.remaining(), 7.5 * Gst.SECOND)

        self.assertTrue(playbin.seek_step(-15))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.remaining(), 10 * Gst.SECOND)

        self.assertTrue(playbin.seek_step(15))
        self.assertEqual(playbin.get_state(), Gst.State.PAUSED)
        self.assertEqual(playbin.remaining(), 0)
