# Copyright 2019 (c) Anna Schumaker.
import curds
import gi
import time
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, GLib

Builder = Gtk.Builder()
Builder.add_from_file("emmental.ui")

UpArrow   = Builder.get_object("up_arrow")
DownArrow = Builder.get_object("down_arrow")
ShowMore  = Builder.get_object("show_more")
Window    = Builder.get_object("window")

class EmmentalApplication(Gtk.Application):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, application_id="org.gtk.emmental", **kwargs)
        self.window  = None
        self.idle_id = None

    def do_activate(self):
        if self.window == None:
            self.window = Window
            self.add_window(self.window)
        self.window.present()

    def do_shutdown(self):
        Gtk.Application.do_shutdown(self)
        if self.idle_id != None:
            GLib.source_remove(self.idle_id)
            self.idle_id = None
        curds.stop()

    def do_startup(self):
        Gtk.Application.do_startup(self)


def can_activate_accel(widget, signal):
    widget.stop_emission_by_name("can-activate-accel")
    return widget.is_visible() and widget.is_sensitive() and \
            not type_focused(Gtk.Entry)

def can_activate_entry(widget, signal):
    widget.stop_emission_by_name("can-activate-accel")
    active = not widget.is_focus()
    ShowMore.set_active(active)
    return active

def main_loop(delay=0.0, iteration_delay=0.0):
    time.sleep(delay)
    while Gtk.events_pending():
        time.sleep(iteration_delay)
        Gtk.main_iteration_do(True)

def notify_loop(delay=0.0, iteration_delay=0.0):
    main_loop(delay, iteration_delay)

def timeout_loop(timeout, iteration_delay=0.0):
    timeout = time.time() + timeout
    while time.time() <= timeout:
        time.sleep(iteration_delay)
        main_loop()

def type_focused(type):
    return isinstance(Window.get_focus(), type)

def show_more_toggled(self, *args):
    active = ShowMore.get_active()
    UpArrow.set_visible(active)
    DownArrow.set_visible(not active)
    curds.notify.notify("show-more", active)

ShowMore.connect("toggled", show_more_toggled)

Application  = EmmentalApplication()
