# Copyright 2019 (c) Anna Schumaker.
from .. import gtk
from .. import playlist
import curds
import os
from gi.repository import GObject, Gtk, GLib

Treeview = gtk.Builder.get_object("node_treeview")


class NodeTreeModel(GObject.GObject, Gtk.TreeModel):
    def __init__(self):
        GObject.GObject.__init__(self)
        self.height = 0
        self.reset()

    def do_get_column_type(self, col):
        if col == 2:
            return int
        return str

    def do_get_iter(self, path):
        plist = curds.playlist.Root.lookup_path(path.get_indices())
        if plist == None or plist.parent == None:
            return (False, None)
        return (True, self.playlist_iter(plist))

    def do_get_n_columns(self):
        return 3

    def do_get_path(self, iter):
        plist = self.iter_playlist(iter)
        return Gtk.TreePath(plist.get_path())

    def do_get_node_height(self, plist, column):
        area = Treeview.get_cell_area(Gtk.TreePath(0), Treeview.get_column(1))
        self.height = max(self.height, area.height)
        return self.height

    def do_get_value(self, iter, column):
        plist = self.iter_playlist(iter)
        if column == 0:
            return plist.icon
        elif column == 1:
            if plist == curds.playlist.current():
                return f"<b>{plist.get_markup()}</b>"
            return plist.get_markup()

        if isinstance(plist, curds.Playlist):
            return -1
        return self.do_get_node_height(plist, column)

    def do_iter_children(self, parent):
        return self.do_iter_nth_child(parent, 0)

    def do_iter_has_child(self, iter):
        return self.iter_n_children(iter) > 0

    def do_iter_n_children(self, iter):
        if (plist := self.iter_playlist(iter)):
            return plist.n_children()
        return 0

    def do_iter_nth_child(self, iter, n):
        if (plist := self.iter_playlist(iter)) != None:
            if (child := plist.nth_child(n)) != None:
                return (True, self.playlist_iter(child))
        return (False, None)

    def do_iter_next(self, iter):
        if (plist := self.iter_playlist(iter)) != None:
            if (next := plist.next_child()) != None:
                iter.user_data = id(next)
                return (True, iter)
        return (False, None)

    def do_iter_parent(self, child):
        plist = self.iter_playlist(child)
        if plist != None and plist.parent != None:
            if plist.parent != curds.playlist.Root:
                return (True, self.playlist_iter(plist.parent))
        return (False, None)

    def iter_playlist(self, iter):
        if iter and iter.user_data > 0:
            return curds.playlist.Root.lookup_byid(iter.user_data)
        return curds.playlist.Root

    def on_first_child(self, parent, path):
        if parent.parent != None:
            self.row_has_child_toggled(Gtk.TreePath(path), self.playlist_iter(parent))

    def on_playlist_changed(self, plist):
        if plist.parent:
            path = Gtk.TreePath(plist.get_path())
            self.row_changed(path, self.playlist_iter(plist))

    def playlist_iter(self, plist):
        if plist == None or plist.parent == None:
            return None
        iter = Gtk.TreeIter()
        iter.user_data = id(plist)
        return iter

    def reset(self):
        curds.notify.register("first-child",      self.on_first_child, idle=True)
        curds.notify.register("playlist-changed", self.on_playlist_changed, idle=True)
