# Copyright 2019 (c) Anna Schumaker.
from . import model
from .. import gtk
from .. import playlist
import curds
import re
from gi.repository import Gtk

Entry     = gtk.Builder.get_object("node_search")
Error     = gtk.Builder.get_object("node_regex_error")
Model     = model.NodeTreeModel()
Filter    = Model.filter_new()
Pattern   = re.compile("")
Popover   = gtk.Builder.get_object("node_regex_popover")
TreeView  = gtk.Builder.get_object("node_treeview")
Selection = TreeView.get_selection()
Separator = gtk.Builder.get_object("node_separator")

Entry.connect("can-activate-accel", gtk.can_activate_entry)
TreeView.set_model(Filter)


def can_select_path(selection, model, path, current):
    child = model.convert_path_to_child_path(path)
    plist = curds.playlist.Root.lookup_path(child)
    return isinstance(plist, curds.Playlist)

Selection.set_select_function(can_select_path)


def on_selection_changed(selection):
    (model, rows) = selection.get_selected_rows()
    if len(rows) > 0:
        child = model.convert_path_to_child_path(rows[0])
        plist = curds.playlist.Root.lookup_path(child)
        playlist.view.switch(plist)

Selection.connect("changed", on_selection_changed)


def on_row_activated(treeview, path, column):
    child  = Filter.convert_path_to_child_path(path)
    plist  = curds.playlist.Root.lookup_path(child)
    if isinstance(plist, curds.Playlist):
        curds.playlist.select(plist)

TreeView.connect("row-activated", on_row_activated)


def on_search_changed(entry):
    global Pattern
    try:
        Pattern = re.compile(entry.get_text(), re.I)
        Filter.refilter()
        Entry.get_style_context().remove_class("warning")
        Popover.popdown()
    except re.error as e:
        Entry.get_style_context().add_class("warning")
        Error.set_text(str(e).capitalize())
        Popover.popup()

Entry.connect("search-changed", on_search_changed)


def visible_func(model, iter, data):
    plist = Model.iter_playlist(iter)
    for n in plist.walk():
        if Pattern.search(n.name) != None:
            return True
        if Pattern.search(curds.sort.normalize(n.name)) != None:
            return True
    return False

Filter.set_visible_func(visible_func)


def on_child_inserted(child, path):
    if isinstance(child, curds.playlist.Library):
        fpath = Filter.convert_child_path_to_path(Gtk.TreePath(path))
        TreeView.expand_to_path(fpath)

curds.notify.register("child-inserted", on_child_inserted, idle=True)


def on_show_more(visible):
    Entry.set_visible(visible)
    Separator.set_visible(visible)

curds.notify.register("show-more", on_show_more)


def reset():
    global Pattern
    TreeView.set_model(None)
    Entry.set_text("")
    Pattern = re.compile("")
    Model.reset()
    Filter.clear_cache()
    on_show_more(False)
    TreeView.set_model(Filter)
    curds.notify.register("show-more",      on_show_more)
    curds.notify.register("child-inserted", on_child_inserted, idle=True)
