# Copyright 2019 (c) Anna Schumaker.
from .. import gtk
import curds
from gi.repository import GObject, Gtk

cols = [ "starred", "upnext", "tracknumber", "title", "length",
         "artist", "album", "discnumber", "date", "genre" ]

class PlaylistModel(GObject.GObject, Gtk.TreeModel, Gtk.TreeDragSource, Gtk.TreeDragDest):
    def __init__(self, playlist):
        GObject.GObject.__init__(self)
        self.playlist = playlist
        self.playlist.visible = True

    def __del__(self):
        self.playlist.visible = False

    def do_drag_data_delete(self, path):
        return path[0] < len(self.playlist)

    def do_drag_data_get(self, path, selection):
        return Gtk.tree_set_row_drag_data(selection, self, path)

    def do_drag_data_received(self, target, selection):
        (ret, model, path) = Gtk.tree_get_row_drag_data(selection)
        if ret == True:
            self.move_track(path, target)
        return ret

    def do_get_column_type(self, col):
        return str

    def do_get_flags(self):
        return Gtk.TreeModelFlags.LIST_ONLY

    def do_get_iter(self, path):
        return self.do_iter_nth_child(None, path.get_indices()[0])

    def do_get_n_columns(self):
        return len(cols)

    def do_get_path(self, iter):
        return Gtk.TreePath((iter.user_data))

    def do_get_value(self, iter, column):
        colname = cols[column]
        track   = self.iter_track(iter)
        if track == None:
            return ""
        if colname == "starred":
            if curds.playlist.Starred.contains(track):
                return "starred"
            return "non-starred"
        elif colname == "upnext":
            if curds.playlist.UpNext.contains(track):
                return "edit-redo"
            return "edit-redo-symbolic"
        return track[colname]

    def do_iter_children(self, parent):
        return (False, None)

    def do_iter_has_child(self, iter):
        return False

    def do_iter_n_children(self, iter):
        if iter != None:
            return 0
        return len(self.playlist)

    def do_iter_nth_child(self, iter, n):
        if iter or n >= len(self.playlist):
            return (False, None)
        iter = Gtk.TreeIter()
        iter.user_data = n
        return (True, iter)

    def do_iter_next(self, iter):
        iter.user_data += 1
        if iter.user_data >= len(self.playlist):
            return (False, None)
        return (True, iter)

    def do_row_draggable(self, path):
        return path[0] < len(self.playlist)

    def do_row_drop_possible(self, path, selection):
        return path[0] < len(self.playlist)

    def move_track(self, old, new):
        self.playlist.move(self.path_track(old), new[0])

        for i in range(min(old[0], new[0]), max(old[0], new[0]) + 1):
            path = Gtk.TreePath(i)
            self.row_changed(path, self.get_iter(path))

    def iter_track(self, iter):
        return self.playlist[iter.user_data]

    def path_track(self, path):
        return self.playlist[path[0]]

    def track_iter(self, track):
        index = self.playlist.index(track)
        if index == None:
            return None
        iter = Gtk.TreeIter()
        iter.user_data = index
        return iter
