# Copyright 2019 (c) Anna Schumaker.
from . import model
from .. import audio
from .. import gtk
import curds
import re
from gi.repository import Gtk

Entry     = gtk.Builder.get_object("track_search")
Error     = gtk.Builder.get_object("track_regex_error")
Filter    = None
Loop      = gtk.Builder.get_object("loop_button")
Model     = None
NextPos   = 0
Pattern   = re.compile("")
Popover   = gtk.Builder.get_object("track_regex_popover")
Random    = gtk.Builder.get_object("random_button")
Runtime   = gtk.Builder.get_object("runtime")
TreeView  = gtk.Builder.get_object("playlist_treeview")
Selection = TreeView.get_selection()
Separator = gtk.Builder.get_object("track_separator")

Entry.connect("can-activate-accel", gtk.can_activate_entry)


def on_loop_toggled(button):
    active = button.get_active()
    button.set_active(Model.playlist.set_loop(active))

Loop.connect("toggled", on_loop_toggled)


def on_random_toggled(button):
    active = button.get_active()
    button.set_active(Model.playlist.set_random(active))

Random.connect("toggled", on_random_toggled)


def on_row_activated(treeview, path, column):
    child = Filter.convert_path_to_child_path(path)
    track = Model.path_track(child)
    Model.playlist.current = child[0]
    audio.play_track(track)

TreeView.connect("row-activated", on_row_activated)


def get_selected_rows():
    (model, rows) = Selection.get_selected_rows()
    return [ Filter.convert_path_to_child_path(p) for p in rows ]

def on_add_remove(plist, path):
    rows   = get_selected_rows()
    add_rm = plist.remove if plist.contains(Model.path_track(path)) else plist.add

    if len(rows) == 0 or path[0] >= rows[-1][0]:
        rows = rows + [ path ]
    elif len(rows) > 0 and path[0] < rows[0][0]:
        rows = [ path ] + rows

    for row in rows:
        add_rm(Model.path_track(row))
    TreeView.queue_draw()
    return add_rm == plist.add

def on_button_press(treeview, event):
    (path, col, x, y) = TreeView.get_path_at_pos(event.x, event.y)
    path = Filter.convert_path_to_child_path(path)
    if col == TreeView.get_column(0):
        on_add_remove(curds.playlist.Starred, path)
        return True
    elif col == TreeView.get_column(1):
        if on_add_remove(curds.playlist.UpNext, path) == True:
            curds.playlist.select(curds.playlist.UpNext)
        return True
    return False

TreeView.connect("button-press-event", on_button_press)


def on_column_clicked(column, index):
    valid = Model.playlist.sort(model.cols[index])
    column.set_sort_indicator(valid)

for i, column in enumerate(TreeView.get_columns()):
    column.connect("clicked", on_column_clicked, i)


def on_search_changed(entry):
    global Pattern
    try:
        Pattern = re.compile(entry.get_text(), re.I)
        Filter.refilter()
        Entry.get_style_context().remove_class("warning")
        Popover.popdown()
    except re.error as e:
        Entry.get_style_context().add_class("warning")
        Error.set_text(str(e).capitalize())
        Popover.popup()

Entry.connect("search-changed", on_search_changed)


def visible_func(plist_model, iter, data):
    track = plist_model.iter_track(iter)
    for col in model.cols[2:]:
        text = track[col]
        if Pattern.search(text) != None:
            return True
        if col in [ "title", "artist", "album", "genre" ]:
            if Pattern.search(curds.sort.normalize(text)) != None:
                return True
    return False


def on_add_track(plist, track, index):
    global NextPos
    if plist == Model.playlist:
        iter = Gtk.TreeIter()
        iter.user_data = NextPos
        Model.row_inserted(Gtk.TreePath(NextPos), iter)
        NextPos += 1

curds.notify.register("add-track", on_add_track, idle=True)


def on_remove_track(plist, track, index):
    global NextPos
    if plist == Model.playlist:
        Model.row_deleted(Gtk.TreePath(index))
        NextPos -= 1
    elif plist == curds.playlist.UpNext and Model.playlist.contains(track):
        TreeView.queue_draw()

curds.notify.register("remove-track", on_remove_track, idle=True)


def set_runtime(text):
    if text != "":
        text = f" {text} "
    Runtime.set_text(text)

def on_playlist_changed(plist):
    if plist == Model.playlist:
        set_runtime(plist.runtime())

curds.notify.register("playlist-changed", on_playlist_changed, idle=True)


def scroll_to(index):
    Selection.unselect_all()
    if index >= 0:
        path = Gtk.TreePath(index)
        Selection.select_path(path)
        TreeView.scroll_to_cell(path, None, True, 0.25, 0)

def on_stream_start():
    plist = Model.playlist
    if plist == curds.playlist.current():
        scroll_to(plist.current)

curds.notify.register("stream-start", on_stream_start)


def on_show_more(visible):
    Entry.set_visible(visible)
    Separator.set_visible(visible)

curds.notify.register("show-more", on_show_more)


def configure_toggle(toggle, sensitive, active):
    toggle.set_active(active)
    toggle.set_sensitive(sensitive)

def switch(plist, force=False):
    global Filter
    global Model
    global NextPos
    if not Model or Model.playlist != plist or force == True:
        Model   = model.PlaylistModel(plist)
        Filter  = Model.filter_new()
        NextPos = len(plist)
        TreeView.set_model(Filter)
        Filter.set_visible_func(visible_func)
        configure_toggle(Random, plist.can_random, plist.random)
        configure_toggle(Loop,   plist.can_loop,   plist.loop)
        set_runtime(plist.runtime())
        scroll_to(plist.current)

switch(curds.playlist.lookup("Collection"))
 

def reset():
    switch(curds.playlist.lookup("Collection"))
    curds.notify.register("add-track", on_add_track, idle=True)
    curds.notify.register("remove-track", on_remove_track, idle=True)
    curds.notify.register("playlist-changed", on_playlist_changed, idle=True)
    curds.notify.register("stream-start", on_stream_start)
    curds.notify.register("show-more", on_show_more)
