# Copyright 2019 (c) Anna Schumaker.
import curds
import os
import rind
import unittest

class TestEmmental(unittest.TestCase):
    def test_import_curds(self):
        self.assertEqual(curds.DataFile,    curds.data.DataFile)
        self.assertEqual(curds.ThreadQueue, curds.threadqueue.ThreadQueue)
        self.assertEqual(curds.Track,       curds.trak.track.Track)

        self.assertEqual(curds.Playlist,         curds.playlist.playlist.Playlist)
        self.assertEqual(curds.playlist.Library, curds.playlist.library.LibraryPlaylist)
        self.assertEqual(curds.playlist.Starred, curds.playlist.lookup("Playlists").lookup("Starred"))

        self.assertIsNotNone(curds.notify)
        self.assertIsInstance(curds.playlist.Root, curds.playlist.root.PlaylistRoot)

    def test_import_rind(self):
        self.assertEqual(rind.Application,   rind.gtk.Application)
        self.assertEqual(rind.audio.Playbin, rind.audio.playbin.Playbin)

        self.assertEqual(rind.node.Model, rind.node.model.NodeTreeModel)
        self.assertEqual(rind.node.View,  rind.node.view.TreeView)

        self.assertEqual(rind.playlist.Model, rind.playlist.model.PlaylistModel)
        self.assertEqual(rind.playlist.View,  rind.playlist.view.TreeView)

    def test_reset_stop_curds(self):
        path  = os.path.join("trier", "Test Album", "01 - Test Track.ogg")
        track = curds.trak.lookup(path)
        curds.playlist.library.library_thread.stop()

        self.assertIsNotNone(track)
        self.assertGreater(len(curds.trak.trak_db), 0)
        self.assertFalse(curds.playlist.library.library_thread.is_alive())

        curds.reset()
        self.assertEqual(curds.trak.trak_db, {})
        self.assertTrue(curds.playlist.library.library_thread.is_alive())
        self.assertEqual(len(curds.playlist.lookup("Collection")), 0)
        self.assertEqual(curds.playlist.lookup("Libraries").n_children(), 0)

        curds.stop()
        self.assertFalse(curds.playlist.library.library_thread.is_alive())
